#include <gdi.h>
#include "ImageControl.h"


CImageControl::CImageControl()
{
}

CImageControl::~CImageControl()
{
	delete iBitmap;
	iWinGroup.Close();	
	CloseWindow();
}

CImageControl*  CImageControl::NewL()
	{
	CImageControl* ctrl = new (ELeave) CImageControl;
	CleanupStack::PushL(ctrl);
	ctrl->ConstructL();
	CleanupStack::Pop(ctrl);
	return ctrl;
	}

void CImageControl::ConstructL()
{
	RWsSession& session = CCoeEnv::Static()->WsSession();
	iWinGroup = RWindowGroup(session);
	TInt res = iWinGroup.Construct((TUint32)&iWinGroup, EFalse);
	User::LeaveIfError(res);

	iWinGroup.SetOrdinalPosition(0, ECoeWinPriorityAlwaysAtFront);
	
	CreateWindowL(iWinGroup);	

	SetRect(TRect(0,0, 0, 0));
    ActivateL();

	iBitmap = 0;
}


void CImageControl::SetBitmapL(TPoint aPos, CFbsBitmap* aBitmap)
	{	
	if (iBitmap)
		{
		delete iBitmap;
		iBitmap = 0;
		}

	CFbsBitmap* bitmap = new (ELeave) CFbsBitmap;
	bitmap->Create(aBitmap->SizeInPixels(), aBitmap->DisplayMode());
		
	//get the GC of this new bitmap
	CFbsBitmapDevice* bitmapDevice = CFbsBitmapDevice::NewL(bitmap);
	CleanupStack::PushL(bitmapDevice);

	CFbsBitGc* bitmapGc;
	bitmapDevice->CreateContext(bitmapGc);
	CleanupStack::PushL(bitmapGc);

	//the copy happens here
	bitmapGc->BitBlt(TPoint(0,0), aBitmap, TRect(aBitmap->SizeInPixels()));

	CleanupStack::PopAndDestroy(2); // bitmapgc, bitmapdevice

	SetExtent(aPos, bitmap->SizeInPixels());

	iBitmap = bitmap;
	DrawDeferred();			
	}

void CImageControl::UnSetBitmap()
	{	
	if (iBitmap)
		{
		SetRect(TRect(0, 0, 0, 0));
		delete iBitmap;
		iBitmap = 0;
		DrawDeferred();
		}
	}

void CImageControl::Draw(const TRect& aRect) const
{
	
	CWindowGc& gc = SystemGc();
	if (iBitmap)
		{		
		gc.BitBlt(TPoint(0,0), iBitmap);
		}
}
