#include <e32std.h>
#include <e32base.h>
#include <fbs.h>
#include <w32std.h>
#include <coemain.h>

#include "Python.h"
#include "symbian_python_ext_util.h"

#include "ImageControl.h"


#define fgimage_type_string "fgimage.type_FGImage"
#define type_FGImage (*(PyTypeObject *)SPyGetGlobalString(fgimage_type_string)) 


// type-definition
typedef struct {
    PyObject_HEAD;
	CImageControl* iImageControl;
} obj_FGImage;


static PyObject* FGImage_set(obj_FGImage* self, PyObject* args);
static PyObject* FGImage_unset(obj_FGImage* self, PyObject* args);


// ctor (factory-function)
static PyObject* fgimage_FGImage(PyObject* /*self*/, PyObject * /*args*/)
{
    obj_FGImage *fgimage = PyObject_New(obj_FGImage, &type_FGImage);

	if (!fgimage)
		{
		return 0;
		}

	fgimage->iImageControl = 0;
	TRAPD(err,
		fgimage->iImageControl = CImageControl::NewL();		
	);

	if (err)
		{
		PyObject_Del(fgimage);		
		return SPyErr_SetFromSymbianOSErr(err);
		}

     return (PyObject*)fgimage;
}

// dtor
static void dealloc_FGImage(obj_FGImage* fgimage)
{   
	delete fgimage->iImageControl;
	fgimage->iImageControl = 0;
    PyObject_Del(fgimage);    
}
 


static const PyMethodDef FGImage_methods[] =
  {
    {"set", (PyCFunction)FGImage_set, METH_VARARGS},
    {"unset", (PyCFunction)FGImage_unset, METH_NOARGS},    
    {NULL, NULL} /* sentinel */
  };

static PyObject *getattr_FGImage(PyObject *self, char *name)
{
  return Py_FindMethod(const_cast<PyMethodDef*>(&FGImage_methods[0]), self, name);
}

static const PyTypeObject type_template_FGImage = {
/*******************************************************/
    PyObject_HEAD_INIT(0)    /* initialize to 0 to ensure Win32 portability */
    0,                 /*ob_size*/
    "fgimage.FGImage",            /*tp_name*/
    sizeof(obj_FGImage), /*tp_basicsize*/
    0,                 /*tp_itemsize*/
    /* methods */
    (destructor)dealloc_FGImage, /*tp_dealloc*/
	 0, /*tp_print*/
    (getattrfunc)getattr_FGImage, /*tp_getattr*/

    /* implied by ISO C: all zeros thereafter */
}; 


// FGImage methods

static PyObject* FGImage_set(obj_FGImage* self, PyObject* args)
{
	PyObject* ptr;	
	TInt x, y;
	

	if (!PyArg_ParseTuple(args, "iiO", &x, &y, &ptr))
	{
		return 0;
	}
	
	if (!PyCObject_Check(ptr))
		{
		PyErr_SetString(PyExc_TypeError, "parameter is not PyCObject(pass graphics.Image()._bitmapapi())");
		return 0;
		}

	// let's hope this cast is valid
	CFbsBitmap* image = (CFbsBitmap*) PyCObject_AsVoidPtr(ptr);	
	
	TRAPD(err,
		self->iImageControl->SetBitmapL(TPoint(x, y), image);
		);

	if (err)
		{
		return SPyErr_SetFromSymbianOSErr(err); 
		}

	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* FGImage_unset(obj_FGImage* self, PyObject* /*args*/)
{
	self->iImageControl->UnSetBitmap();
	Py_INCREF(Py_None);
	return Py_None;
}


// ctor (factory-function)
static PyObject* fgimage_getpixel(PyObject* /*self*/, PyObject * args)
{
	PyObject* ptr;	
	TInt x, y;
	

	if (!PyArg_ParseTuple(args, "iiO", &x, &y, &ptr))
	{
		return 0;
	}
	
	if (!PyCObject_Check(ptr))
		{
		PyErr_SetString(PyExc_TypeError, "parameter 3 is not PyCObject(pass graphics.Image()._bitmapapi())");
		return 0;
		}

	// let's hope this cast is valid
	CFbsBitmap* image = (CFbsBitmap*) PyCObject_AsVoidPtr(ptr);	
	
    TRgb rgb;
    TPoint coords(x, y);
    image->GetPixel(rgb, coords);

    // s60/python uses 0xrrggbb format
    TInt color;

    color = rgb.Red() << 16 | rgb.Green() << 8 | rgb.Blue();

    return Py_BuildValue("i", color);
}

static const PyMethodDef fgimage_methods[] =
{
	{"FGImage", (PyCFunction)fgimage_FGImage, METH_NOARGS},		
    {"getpixel", (PyCFunction)fgimage_getpixel, METH_VARARGS},	
	{0, 0} /* sentinel */
};

DL_EXPORT(void) initfgimage()
{
	// work done here:
	// 1. init module
    // 2. create type object for our FGImage type


	PyObject* module = Py_InitModule("fgimage", (PyMethodDef*) fgimage_methods);
	if (!module)
		return;

	PyTypeObject *fgimageTypeObject = PyObject_New(PyTypeObject, &PyType_Type);
	if (!fgimageTypeObject)
		return;

    *fgimageTypeObject = type_template_FGImage;

	TInt err = SPyAddGlobalString(fgimage_type_string, (PyObject *)fgimageTypeObject);
    if (0 != err) // 0 is success
		{
		PyObject_Del(fgimageTypeObject);
		PyErr_SetString(PyExc_Exception, "SPyAddGlobalString failed");
		return;
		}
    
    // notice that the this uses the macro defined in the beginning of file
    type_FGImage.ob_type = &PyType_Type; 
}

GLDEF_C TInt E32Dll(TDllReason)
{
	return KErrNone;
}